'use strict';


function MysqlProbe(agent) {
  this.agent = agent;

  this.packages = ['mysql'];
}
exports.MysqlProbe = MysqlProbe;



MysqlProbe.prototype.attach = function(obj) {
  var self = this;

  if(obj.__appdynamicsProbeAttached__) return;
  obj.__appdynamicsProbeAttached__ = true;

  var proxy = self.agent.proxy;
  var profiler = self.agent.profiler;
  var type = 'MySQL';

  ['createClient', 'createConnection'].forEach(function(createCmd) {
    proxy.after(obj, createCmd, function(obj, args, ret) {
      var client = ret;
      var config = (createCmd === 'createClient' ? client : client.config);
      if(!config) return;

      var identifyingProperties = {
        Host : config.host,
        Port : config.port,
        Vendor : "MYSQL"
      };
      var database = config.database !== '' ? config.database : undefined;
      if(database) identifyingProperties['Database'] = database;

      proxy.before(client, 'query', function(obj, args) {
        var command = args.length > 0 ? args[0] : undefined;
        var params = args.length > 1 && Array.isArray(args[1]) ? args[1] : undefined;

        var time = profiler.time();
        var trace = profiler.stackTrace();
        var exitCall = profiler.createExitCall(time, {
          exitType: 'EXIT_DB', 
          backendName: 'MySQL', 
          identifyingProperties: identifyingProperties,
          user: config.user,
          command: profiler.truncate(command),
          commandArgs: profiler.truncate(params),
          stackTrace: trace,
          label: config.host + ':' + config.port + ' - MySQL',
          isSql: true
        });

        proxy.callback(args, -1, function(obj, args) {
          if (exitCall) {
            if(!time.done()) return;
            var error = proxy.getErrorObject(args);
            profiler.addExitCall(time, exitCall, error);
          }
        });
      });
    });
  });
};
