'use strict';


function PgProbe(agent) {
  this.agent = agent;

  this.packages = ['pg'];
}
exports.PgProbe = PgProbe;



PgProbe.prototype.attach = function(obj) {
  var self = this;

  if(obj.__appdynamicsProbeAttached__) return;
  obj.__appdynamicsProbeAttached__ = true;

  var proxy = self.agent.proxy;
  var profiler = self.agent.profiler;

  function probe(obj) {
    if(obj.__appdynamicsProbeAttached__) return;
    obj.__appdynamicsProbeAttached__ = true;

    // Callback API
    proxy.before(obj, 'query', function(obj, args, ret) {
      var time = profiler.time();
      var exitCall;

      var client = obj;
      var command = args.length > 0 ? args[0] : undefined;
      var params = args.length > 1 && Array.isArray(args[1]) ? args[1] : undefined;
      var database = client.database ? client.database : undefined;
      var trace = profiler.stackTrace();

      exitCall = profiler.createExitCall(time, {
        exitType: 'EXIT_DB', 
        backendName: 'PostgreSQL', 
        identifyingProperties: {
          Host : client.host,
          Port : client.port,
          Database : database,
          Vendor : "POSTGRESQL"
        },
        command: truncate(profiler, command),
        commandArgs: profiler.truncate(params),
        user: client.user,
        stackTrace: trace,
        label: client.host + ':' + client.port + ' - PostgreSQL',
        isSql: true
      });

      proxy.callback(args, -1, function(obj, args) {
        if (exitCall) {
          if(!time.done()) return;
          var error = proxy.getErrorObject(args);
          profiler.addExitCall(time, exitCall, error);
        }
      });
    });


    // Evented API
    proxy.after(obj, 'query', function(obj, args, ret) {
      // If has a callback, ignore
      if(args.length > 0 && typeof args[args.length - 1] === 'function') return;
      
      var time = profiler.time();
      var exitCallInfo, exitCall;
      var error;

{
      var client = obj;
      var command = args.length > 0 ? args[0] : undefined;
      var params = args.length > 1 && Array.isArray(args[1]) ? args[1] : undefined;
      var database = client.database ? client.database : undefined;
      var trace = profiler.stackTrace();

      exitCallInfo = {
        exitType: 'EXIT_DB', 
        backendName: 'PostgreSQL', 
        identifyingProperties: {
          Host : client.host,
          Port : client.port,
          Database : database,
          Vendor : "POSTGRESQL"
        },
        command: truncate(profiler, command),
        commandArgs: profiler.truncate(params),
        user: client.user,
        stackTrace: trace,
        label: client.host + ':' + client.host + ' - ' + 'PostgreSQL',
        isSql: true
      };
}
      exitCall = profiler.createExitCall(time, exitCallInfo);

      proxy.before(ret, 'on', function(obj, args) {
        var event = args[0];

        if(event !== 'end' && event !== 'error') return;

        proxy.callback(args, -1, function(obj, args) {
          if(event === 'error') {
            error = proxy.getErrorObject(args);
            return;
          }

          if (exitCall) {
            if(!time.done()) return;
            profiler.addExitCall(time, exitCall, error);
          }
        });
      });
    });
  }


  // Native, reinitialize probe
  proxy.getter(obj, 'native', function(obj, ret) {
    proxy.after(ret, 'Client', function(obj, args, ret) {
      probe(ret.__proto__);
    });
  });

  probe(obj.Client.prototype);
};


function truncate(profiler, str) {
  if(str && typeof(str) === 'object') {
    str = str.text;
  }

  return profiler.truncate(str);
}
