'use strict';


function TimePromise(agent, transactionInfo) {
  this.agent = agent;

  this.info = transactionInfo;
  this.stackTrace = undefined;
  this.time = undefined;

  // optional callbacks:
  this.beforeExitCall =
  this.exitCallCompleted = null;
}

exports.TimePromise = TimePromise;


TimePromise.prototype.start = function() {
  var self = this;
  var profiler = self.agent.profiler;

  self.time = profiler.time(true);
  self.stackTrace = profiler.stackTrace();
  self.transaction = profiler.startTransaction(
    self.time, self.info, 'NODEJS_API');
  self.transaction.api = this;
};


TimePromise.prototype.resume = function(err) {
  if (!this.time) {
    throw new Error('transaction not started');
  }
  this.agent.thread.resume(this.time.threadId);
  this.transaction.touched = this.agent.system.millis();
};


TimePromise.prototype.end = function(err) {
  var self = this;
  var profiler = self.agent.profiler;

  if(!self.time.done()) return;

  if (err) {
    self.transaction.error = err;
    self.transaction.stackTrace = profiler.formatStackTrace(err);
    self.transaction.statusCode = self.transaction.statusCode || 500;
  }

  profiler.endTransaction(self.time, self.transaction);
};

TimePromise.prototype.startExitCall = function(exitCallInfo) {
  var profiler = this.agent.profiler;
  var callback = this.beforeExitCall;
  var time = profiler.time();

  try {
    this.beforeExitCall = null; // bypass callback for explicit create
    return profiler.createExitCall(time, exitCallInfo);
  } finally {
    this.beforeExitCall = callback; // restore any callback
  }
};

TimePromise.prototype.endExitCall = function(exitCall, error) {
  var time = exitCall.time;
  if (!time.done()) return;
  this.agent.profiler.addExitCall(exitCall.time, exitCall, error);
};

TimePromise.prototype.createCorrelationInfo = function(exitCall) {
  var header = this.agent.correlation.newCorrelationHeader();
  header.build(this.transaction, exitCall);
  return header.getStringHeader();
};
