'use strict';

var uuid = require('node-uuid');
var EumCookie = require('./eum-cookie').EumCookie;


function Eum(agent) {
  this.agent = agent;

  this.enabled = false;

  // GUID generation
  this.uuidBase = uuid.v4();
  this.nextEumCookieId = 1;

  // constants
  this.ADRUM_MASTER_COOKIE_NAME = "ADRUM_BT";
  this.ADRUM_PREFIX = "ADRUM_";
  this.CLIENT_REQUEST_GUID_KEY = "clientRequestGUID";
  this.BT_ID_KEY = "btId";
  this.BT_ART_KEY = "btERT";
  this.BT_DURATION_KEY = "btDuration";
  this.BACKEND_SNAPSHOT_TYPE_KEY = "backendSnapshotType";
  this.HAS_ENTRY_POINT_ERRORS_KEY = "hasEntryPointErrors";
}
exports.Eum = Eum;


Eum.prototype.init = function() {
	var self = this;

  self.agent.on('configUpdated', function() {
    self.enabled = self.agent.configManager.getConfigValue("eumConfig.enabled");
  });
};


Eum.prototype.generateGuid = function() {
  var self = this;

  return (self.uuidBase + (self.nextEumCookieId++));
};


Eum.prototype.newEumCookie = function(transaction, request, response, isHttps) {
	var self = this;

	return new EumCookie(self.agent, transaction, request, response, isHttps);
};

